;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;+
; NAME:checkfield
;
; PURPOSE:en entree de plt, pltz, pltt et plt1d, verifie que le
; champ donne a bien une taille compatible avec le domaine et fait au
; besoin les moyennes pour ressortir en fin de fonction un tableau 2d
; si on fait un plot du type: 'xy', 'xz', 'xt', 'yz', 'yt', 'zt' ou un
; tableau 1d si on fait un plot du type 'x', 'y', 'z', 't'.
;
; CATEGORY:en entree de plt, pltz, pltt et plt1d
;
; CALLING SEQUENCE:res=checkfield(field, procedure)
; 
; INPUTS:
;        filed: un champ recomdant aux criteres de litchamp.pro, cf.
;        IDL>xhelp,'litchamp'
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
; COMMON BLOCKS:common.pro
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:Sebastien Masson (smasson@lodyc.jussieu.fr)
;                      08/02/2000
;-
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
FUNCTION checkfield, field, procedure, TYPE = type, BOITE = boite, DIREC = direc, NOQUESTION = noquestion, VECTEUR = vecteur, _EXTRA = ex
;--------------------------------------------------------------
@common
;--------------------------------------------------------------
; I1) lecture du champ
;--------------------------------------------------------------
   if n_elements(field) EQ 0 then return, report('field undefined')
   z2d = litchamp(field)
   nan = total(finite(z2d,/nan)) < 1
;---------------------------------------------------------------
; redefinition du domaine 
;---------------------------------------------------------------
   minprof = 0
   profdefault = 200
   if keyword_set(boite) then BEGIN 
;
;      if n_elements(integration3d) EQ 0 THEN integration3d = n_elements(boite) ne 4
;
      neleboitein = N_Elements(Boite)
      Case neleboitein Of
         1:boite=[lon1, lon2, lat1, lat2, minprof,boite[0]]
         2:boite=[lon1, lon2, lat1, lat2, boite[0],boite[1]]
         4:$ 
          if strpos(type, 'z') NE -1 THEN boite=[Boite, minprof, profdefault] $
         ELSE boite=[Boite, prof1, prof2]
         5:boite=[Boite[0:3], minprof, Boite[4]]
         6:boite=Boite
         Else: return, report('Mauvaise Definition de Boite')
      ENDCASE
   ENDIF ELSE neleboitein = 0
; la procedure domdef determine les elements qui sont a l''interieur
; de la boite. 
; si on fait un plot contenant l''axe z:
; Suivant l''axe z si on veut par ex faire un dessin
; entre 0 et 1000 il se peut que l''on selectionne les niveaux
; correspondants aux profondeurs comprises entre 0 et 900m (vu la
; discretisation assez lache de cet axe qd on atteint des profondeurs
; elevees). Pour ne pas que le dessin s''arrete a 900 mais bien a
; 1000, on va aller chercher le niveau au dessous de 1000, d''ou la
; manip suivante sur la boite: l''approfondir de 1 niveau sur la
; verticale (si possible) sans changer les y range. 
   if strpos(type, 'z') NE -1 THEN BEGIN
      if NOT keyword_set(boite) then boite = 1.*[lon1,lon2,lat1,lat2,minprof, profdefault]
      !y.range = [boite[5], boite[4]] ;on garde les yranges (axe z) avant de changer la boite.
      profmax = boite[5]
      if vargrid EQ 'W' then gdep = gdepw ELSE gdep = gdept
; check with vertical grid limits (nearest level)
      gwork = gdep
; check the increse or decrese of the z axis
      IF gwork[1] LE gwork[0] THEN gwork = reverse(gdep, 1)
      niveauprof = where(gwork ge boite[5]) & niveauprof = niveauprof[0]
      if niveauprof NE -1 then boite[5] = gwork[niveauprof]+1
   ENDIF
   if keyword_set(boite) then BEGIN
      if keyword_set(vecteur) then grillechoice = [vargrid, 'T', 'U', 'V'] ELSE grillechoice = [vargrid, 'T']
      if neleboitein LE 2 then domdef, boite, GRILLE=grillechoice ELSE domdef, boite, GRILLE=grillechoice, _extra = ex
   ENDIF
;--------------------------------------------------------------
; verification de la taille du tableau d''entree et de la valeur de type
;--------------------------------------------------------------
   grille, mask, glam, gphi, gdep, nx, ny,nz,premierx,premiery,premierz,dernierx,derniery,dernierz
   taille=size(z2d)
   case taille[0] of
;--------------------------------------------------------------
      0:BEGIN 
         if z2d EQ -1 then $
          return, report('Attention le tableau est egale a -1. Probleme lors de la lecture...') $
         ELSE return, report('Attention le tableau est un scalaire') 
      END 
;--------------------------------------------------------------
      1:BEGIN
         nele = n_elements(z2d)
         case type of
            't':if jpt NE nele then $
             return, report('jpt: '+strtrim(jpt, 1)+' n''est pas egale au nombres d''elements du vecteur: '+strtrim(nele, 1))
            'x':BEGIN 
               case nele of
                  jpi:z2d = z2d[premierx:dernierx]
                  nx:
                  ELSE:return, report('nx: '+strtrim(nx, 1)+' n''est pas egale au nombres d''elements du vecteur: '+strtrim(nele, 1))
               endcase
            end
            'y':BEGIN 
               case nele of
                  jpj:z2d = z2d[premiery:derniery]
                  ny:
                  ELSE:return, report('ny: '+strtrim(ny, 1)+' n''est pas egale au nombres d''elements du vecteur: '+strtrim(nele, 1))
               endcase
            END
            'z':BEGIN 
               case nele of
                  jpk:z2d = z2d[premierz:dernierz]
                  nz:
                  ELSE:return, report('nz: '+strtrim(nz, 1)+' n''est pas egale au nombres d''elements du vecteur: '+strtrim(nele, 1))
               endcase
            END
            ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 1d en entree: '+strtrim(nele, 1))
         ENDCASE
      END 
;--------------------------------------------------------------
      2:BEGIN
         case type of
            'x':BEGIN
               case taille[2] of
                  ny:direc = 'y'
                  jpj:direc = 'y'
                  jpt:direc = 't'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 2d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            end
            'y':BEGIN
               case 1 of
                  taille[1] EQ jpi OR taille[1] EQ nx:direc = 'x'
                  taille[2] EQ jpt:direc = 't'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 2d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
            'z':BEGIN
               if taille[2] EQ jpt then direc = 't' $
               ELSE return, report('imcompatibilite entre type: '+type+' et la taille du tableau 2d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
            END
            't':BEGIN
               case 1 of
                  taille[1] EQ jpi OR taille[1] EQ nx:BEGIN
                     direc = 'x'
                     timearray = 1
                  END
                  jpt EQ 1:BEGIN
                     ras = report('Field average: '+strtrim(moyenne(z2d, 'xy', boite = boite, NAN = nan, _extra = ex), 1)+' in box: '+tostr(boite))
                     return,  -1
                  END
                  ELSE: return, report('imcompatibilite entre type: '+type+' et la taille du tableau 2d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               ENDCASE
            END
            'xy':BEGIN 
               case 1 of
                  taille[1] eq  nx and taille[2] eq  ny:
                  taille[1] eq jpi and taille[2] eq jpj:z2d=z2d[premierx:dernierx, premiery:derniery]
                  else:return, report('Probleme d''adequation entre les tailles du domaine nx*ny '+strtrim(nx, 1)+'*'+strtrim(ny, 1)+' et du tableau '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
               if vargrid EQ 'W' then niveau = premierzw+1 ELSE niveau = premierzt+1
            END
            'xz':BEGIN 
               case 1 of
                  taille[1] eq jpi and taille[2] eq jpk:z2d = z2d[premierx:dernierx,premierz:dernierz]
                  taille[1] eq nx and taille[2] eq jpk:z2d = z2d[*,premierz:dernierz]
                  taille[1] eq nx and taille[2] eq nz:
                  else:return, report('Probleme d''adequation entre les tailles du domaine nx*nz '+strtrim(nx, 1)+'*'+strtrim(nz, 1)+'et du tableau'+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
            'yz':BEGIN 
               case 1 of
                  taille[1] eq jpj and taille[2] eq jpk:z2d = z2d[premiery:derniery,premierz:dernierz]
                  taille[1] eq ny and taille[2] eq jpk:z2d = z2d[*,premierz:dernierz]
                  taille[1] eq ny and taille[2] eq nz:
                  else:return, report('Probleme d''adequation entre les tailles du domaine ny*nz '+strtrim(ny, 1)+'*'+strtrim(nz, 1)+'et du tableau'+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
            'xt':BEGIN
               case 1 of
                  taille[1] eq jpi and taille[2] eq jpt:z2d = z2d[premierx:dernierx,*]
                  taille[1] eq nx and taille[2] eq jpt:
                  else:return, report('Probleme d''adequation entre les tailles du domaine nx*jpt '+strtrim(nx, 1)+'*'+strtrim(jpt, 1)+'et du tableau'+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
            'yt':BEGIN
               case 1 of
                  taille[1] eq jpj and taille[2] eq jpt:z2d = z2d[premiery:derniery,*]
                  taille[1] eq ny and taille[2] eq jpt:
                  else:return, report('Probleme d''adequation entre les tailles du domaine ny*jpt '+strtrim(ny, 1)+'*'+strtrim(jpt, 1)+'et du tableau'+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
            'zt':BEGIN
               case 1 of
                  taille[1] eq jpk and taille[2] eq jpt:z2d = z2d[premierz:dernierz,*]
                  taille[1] eq nz and taille[2] eq jpt:
                  else:return, report('Probleme d''adequation entre les tailles du domaine nz*jpt '+strtrim(nz, 1)+'*'+strtrim(jpt, 1)+'et du tableau'+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1))
               endcase
            END
         endcase
      END
;--------------------------------------------------------------
      3:BEGIN
         case type of
            'x':BEGIN
               case taille[3] of
                  nz:direc = 'yz'
                  jpk:direc = 'yz'
                  jpt:direc = 'yt'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
            'y':BEGIN
               case taille [3] of
                  nz:direc = 'xz'
                  jpk:direc = 'xz'
                  jpt:direc = 'xt'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
            'z':BEGIN
               case taille [3] of
                  nz:direc = 'xy'
                  jpk:direc = 'xy'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
            't':BEGIN
               IF jpt EQ 1 THEN BEGIN 
                  ras = report('Field average: '+strtrim(moyenne(z2d, 'xyz', boite = boite, NAN = nan, _extra = ex), 1)+' in box: '+tostr(boite))
                  return,  -1
               ENDIF ELSE BEGIN
                  direc = 'xy'
                  timearray = 1
               ENDELSE
            END
            'xy':BEGIN
               case 1 of
                  taille[3] EQ jpt:BEGIN
                     direc = 't'
                     if vargrid EQ 'W' then niveau = premierzw+1 ELSE niveau = premierzt+1
                  END
                  taille[3] EQ nz OR taille[3] EQ jpk:BEGIN 
                     if keyword_set(boite) OR keyword_set(noquestion) then begin
                        direc = 'z'
                        if vargrid EQ 'W' then niveau = premierzw+1 ELSE niveau = premierzt+1
                     ENDIF ELSE BEGIN
                        if vargrid EQ 'W' then premierz = premierzw ELSE premierz = premierzt
                        if NZ NE 1 then begin
                           niveau = xquestion('Le tableau d''entree est un tableau 3d,' $
                                              +'!C a quel niveau faut-il faire le graphique?', '1', /chkwidget)
                           niveau = 1 > fix(niveau) < (jpk-premierz)
                           case 1 of
                              taille[1] EQ nx AND taille[2] EQ ny: $
                               z2d=z2d[*, *,niveau-1]
                              taille[1] EQ jpi AND taille[2] EQ jpj: $
                               z2d=z2d[premierx:dernierx, premiery:derniery,niveau-1]
                              ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
                           endcase
                        ENDIF ELSE niveau = 1
                        niveau = premierz+niveau
                     endelse    
                  END
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
            'xz':BEGIN
               case 1 of
                  (taille[2] EQ ny OR taille[2] EQ jpj) AND taille[3] EQ jpk:BEGIN
                     z2d = z2d[*, *, premierz:dernierz]
                     direc = 'y'
                  END
                  (taille[2] EQ ny OR taille[2] EQ jpj) AND taille[3] EQ nz:direc = 'y'
                  taille[3] EQ jpt:direc = 't'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
            'yz':BEGIN
               case 1 of
                  (taille[1] EQ nx OR taille[1] EQ jpi) AND taille[3] EQ jpk:BEGIN
                     z2d = z2d[*, *, premierz:dernierz]
                     direc = 'x'
                  END
                  (taille[1] EQ nx OR taille[1] EQ jpi) AND taille[3] EQ nz:direc = 'x'
                  ELSE:return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
               endcase
            END
         'xt':BEGIN & direc = 'y' & timearray = 1 & end
         'yt':BEGIN & direc = 'x' & timearray = 1 & end
            'zt':return, report('imcompatibilite entre type: '+type+' et la taille du tableau 3d en entree: '+strtrim(taille[1], 1)+'*'+strtrim(taille[2], 1)+'*'+strtrim(taille[3], 1))
         ENDCASE
      end
;--------------------------------------------------------------
      4:BEGIN 
         case type of
            'x':direc = 'yzt'
            'y':direc = 'xzt'
            'z':direc = 'xyt'
            't':direc = 'xyz'
            'xy':BEGIN 
               if keyword_set(boite) OR keyword_set(noquestion) then begin
                  if vargrid EQ 'W' then niveau = premierzw+1 ELSE niveau = premierzt+1
                  direc = 'zt'
               ENDIF ELSE BEGIN 
                  if vargrid EQ 'W' then premierz = premierzw ELSE premierz = premierzt
                  if NZ NE 1 then begin
                     niveau = xquestion('Le tableau d''entree est un tableau 4d,' $
                                        +'!C a quel niveau faut-il faire le graphique?', '1', /chkwidget)
                     niveau = 1 > fix(niveau) < (jpk-premierz)
                     z2d=reform(z2d[*,*,niveau-1, *], /over)
                  ENDIF ELSE niveau = 1
                  niveau = premierz+niveau
                  direc = 't'
               ENDELSE
            END
            'xz':direc = 'yt'
            'yz':direc = 'xt'
            'xt':direc = 'yz'
            'yt':direc = 'xz'
            'zt':direc = 'xy'
         endcase
      END
   ENDCASE
;--------------------------------------------------------------
   if keyword_set(direc) then begin
      if strpos(direc, 't') NE -1 OR keyword_set(timearray) OR taille[0] EQ 4 then $
       z2d = grossemoyenne(z2d, direc, boite = boite, NAN = nan, /nodomdef, _extra = ex) $
      ELSE z2d = moyenne(z2d, direc, boite = boite, NAN = nan, /nodomdef, _extra = ex)
   ENDIF
;--------------------------------------------------------------
;--------------------------------------------------------------
;--------------------------------------------------------------
;--------------------------------------------------------------
   return, z2d
end
